@@ -7,6 +7,7 @@
 //
 
 #include "myPAVAalgorithm.cpp"
+#define SMALLNUMBER 0.00000000001;
 
 double max(double a, double b){
     if(a > b) return(a);
@@ -105,7 +106,6 @@ void Rvec2eigen(SEXP r_vec, Eigen::VectorXd &e_vec){
 }
 
 
-
 double ic_dloglogistic(double x, double a, double b){
     double x_a = x/a;
     double x_a_b = pow(x_a, b);
@@ -117,6 +117,13 @@ double ic_ploglogistic(double x, double a, double b){
     return( 1/ (1 + pow(x/a, -b)));
 }
 
+double ic_qloglogistic(double p, double a, double b){
+    double expVal = -1.0/b;
+    double odds = (1.0 - p)/p;
+    double ans = a * pow(odds, expVal);
+    return(ans);
+}
+
 double ic_dlnorm(double x, double mu, double s){
     double denom = x * s * sqrt(2 * PI);
     double expPart = pow((log(x) - mu), 2) / (2 * s * s);
@@ -129,6 +136,42 @@ double ic_plnorm(double x, double mu, double s){
 }
 
 
+
+
+double ic_dgeneralgamma(double x, double mu, double s, double Q){
+    if(Q == 0){
+        return(ic_dlnorm(x, mu, s));
+    }
+    double y = log(x);
+    double w = ((y - mu)/s);
+    double Q_inv = 1/(Q * Q);
+    double logAns = -log(s*x) + log(abs(Q)) + Q_inv * log(Q_inv) + Q_inv * (Q*w - exp(Q*w)) - Rf_lgammafn(Q_inv);
+    return(exp(logAns));
+}
+
+double ic_pgeneralgamma(double q, double mu, double s, double Q){
+    if(Q == 0){ return(ic_plnorm(q, mu, s));}
+    double y = log(q);
+    double w = ((y-mu)/s);
+    double Q_inv = 1 / (Q*Q);
+    double expnu = exp(Q*w) * Q_inv;
+    double ans;
+    if(Q > 0){ ans = 1 - pgamma(expnu, Q_inv, 1.0,  0, 0);}
+    else{ ans = pgamma(expnu, Q_inv, 1.0, 0, 0);}
+    return(ans);
+}
+
+double ic_qgeneralgamma(double p, double mu, double s, double Q){
+    if(Q == 0){ return(qlnorm(p, mu, s, 1, 0)); }
+    double Q2 = (Q*Q);
+    double Q_inv = 1/Q2;
+    double part2 = s * (log(Q2 * qgamma(p, Q_inv, 1.0, 0, 0)) / Q);
+    double ans = exp(mu + part2);
+    return(ans);
+}
+
+
+
 void pavaForOptim(vector<double> &d1, vector<double> &d2, vector<double> &x, vector<double> &prop_delta){
     int k = d1.size();
     int d2_size = d2.size();
@@ -191,6 +234,42 @@ extern "C"{
 
 
 
+extern "C"{
+    SEXP fastNumericInsert(SEXP newVals, SEXP target, SEXP indices){
+        int k1 = LENGTH(newVals);
+        int k2 = LENGTH(indices);
+        int t_k = LENGTH(target);
+        
+        int* c_inds = INTEGER(indices);
+        double* c_targ = REAL(target);
+        double* nv = REAL(newVals);
+        
+        for(int i = 0; i < k2; i++){
+            if(c_inds[i] > t_k){
+                Rprintf("error: index too large for target in fastNumericInsert\n");
+                return(target);
+            }
+            if(c_inds[i] < 1){
+                Rprintf("error: index less than 1 for target in fastNumericInsert\n");
+                return(target);
+            }
+        }
+        
+        if(k1 == 1){
+            double onlyVal = nv[0];
+            for(int i = 0; i < k2; i++){
+                c_targ[c_inds[i] - 1] = onlyVal;
+            }
+        }
+        else{
+            for(int i = 0; i < k2; i++){
+                c_targ[c_inds[i] - 1] = nv[i];
+                }
+            }
+    return(target
+           );
+    }
+}
 
 
 //Added for bivariate NPMLE
